Package: biocViews
Title: Categorized views of R package repositories
Description: Infrastructure to support 'views' used to classify
    Bioconductor packages. 'biocViews' are directed acyclic
    graphs of terms from a controlled vocabulary. There are three
    major classifications, corresponding to 'software', 'annotation',
    and 'experiment data' packages.
biocViews: Infrastructure
URL: http://bioconductor.org/packages/biocViews
BugReports: https://github.com/Bioconductor/biocViews/issues
Version: 1.80.0
License: Artistic-2.0
Authors@R: c(
    person("Vincent", "Carey", role = "aut"),
    person("Benjamin", "Harshfield", role = "aut"),
    person("Seth", "Falcon", role = "aut"),
    person("Sonali", "Arora", role = "aut"),
    person("Lori", "Shepherd", role = "aut"),
    person("Bioconductor Package", "Maintainer", ,
        "maintainer@bioconductor.org", "cre"))
Depends: R (>= 3.6.0)
Imports: Biobase, graph (>= 1.9.26), methods, RBGL (>= 1.13.5), tools,
        utils, XML, RCurl, RUnit, BiocManager
Suggests: BiocGenerics, BiocPkgTools, knitr, commonmark, BiocStyle
Collate: AllClasses.R AllGenerics.R as-methods.R htmlDoc-methods.R
        htmlFilename-methods.R htmlValue-methods.R show-methods.R
        getPackNames.R packageDetails.R pump.R repository.R showvoc.R
        getPackageNEWS.R validation_tests.R recommendBiocViews.R
        dump_concept.R build_dbs.R
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/biocViews
git_branch: RELEASE_3_23
git_last_commit: 3bbfb8e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:54:04 UTC; biocbuild
Author: Vincent Carey [aut],
  Benjamin Harshfield [aut],
  Seth Falcon [aut],
  Sonali Arora [aut],
  Lori Shepherd [aut],
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
