import(methods)

importMethodsFrom(graph, acc, adj, edges, inEdges, nodes, subGraph)

importFrom(Biobase, copySubstitute)

importFrom(RBGL, sp.between)

importFrom(tools, write_PACKAGES, package_dependencies)

importFrom(BiocManager, repositories)

importFrom(stats, complete.cases, setNames)

importFrom(utils, download.file, Stangle, available.packages,
           capture.output, contrib.url, data, file_test, head,
           packageDescription, readCitationFile, untar)

importMethodsFrom(XML, saveXML)

importFrom(XML, xmlNode, xmlOutputDOM, xmlTree, htmlParse, xpathApply, xmlValue)

importFrom(RCurl, getURL)

importFrom(RUnit, checkTrue)

exportClasses("Htmlized",

              "PackageDetail", "pdAuthorMaintainerInfo", "pdVignetteInfo",
              "pdDownloadInfo", "pdDetailsInfo", "pdDescriptionInfo",
              "pdVigsAndDownloads",

              "RepositoryDetail", "rdPackageTable",

              "BiocView", "bvTitle", "bvPackageTable", "bvSubViews",
              "bvParentViews")

exportMethods("coerce", "show", "htmlDoc", "htmlValue", "htmlFilename")

export("writeBiocViews", "getBiocViews", "write_VIEWS", "write_REPOSITORY",
       "genReposControlFiles", "extractVignettes", "extractManuals",
       ## build db functions
       "build_db_from_source", "build_meta_aliases_db", "build_meta_rdxrefs_db",
       ##
       "extractCitations", "getCurrentbiocViews",
       "extractNEWS", "extractHTMLDocuments", "extractTopLevelFiles",
       "writeRepositoryHtml", "writePackageDetailHtml", "getSubTerms",
       "getBiocSubViews", "validate_bioc_views", "writeTopLevelView",
       "writeHtmlDoc", "write_SYMBOLS", "writeRFilesFromVignettes",
       "getPackageNEWS", "printNEWS","recommendBiocViews",
       "recommendPackages", "guessPackageType", "dump_concept")
