[<img src="https://www.bioconductor.org/images/logo/jpg/bioconductor_logo_rgb.jpg" width="200" align="right"/>](https://bioconductor.org/)

**biocViews** is an R/Bioconductor package that implements the infrastructure to support _views_ used to classify Bioconductor packages.

See https://bioconductor.org/packages/biocViews for more information including how to install the release version of the package (please refrain from installing directly from GitHub).

