setGeneric("htmlDoc", function(object, ...) standardGeneric("htmlDoc"))

setGeneric("htmlValue", function(object) standardGeneric("htmlValue"))

setGeneric("htmlFilename",
           function(object, ...) standardGeneric("htmlFilename"))



